import json

from rest_framework.exceptions import ValidationError
from datetime import datetime
from contextlib import suppress
from ..helpers.notification import NotificationHelper
from ..services.abstract import AbstractServiceProvider
from ..config.settings import (SERVICE_NOTIFICATION_HOST, SERVICE_NOTIFICATION_SECRET_KEY)


# Create your services here.


class NotificationService(AbstractServiceProvider):
    URL_POST_WAITING = 'notification/waiting/'
    URL_POST_WAITING_BULK = 'notification/waiting/bulk/'

    def __init__(self):
        super().__init__()

        self.host = SERVICE_NOTIFICATION_HOST
        self.key = SERVICE_NOTIFICATION_SECRET_KEY
        self.headers = self.default_headers

        # Authorize
        self.auth()

    @staticmethod
    def _set_method_payload(**kwargs):
        method = kwargs.get('method', 'email')
        recipient = kwargs.get('recipient', '')
        sender = kwargs.get('sender', '')
        subject = str(kwargs.get('subject', ''))
        body = kwargs.get('body', '')
        internal = kwargs.get('internal', False)

        # Compose payload
        payload = {
            "is_internal_recipient": internal,
            "recipient": recipient,
            "delivery_method": method,
            "message_subject": subject,
            "message_body": str(body)
        }

        if sender != '':
            payload.update({'sender': sender})

        return payload

    def send(self, **kwargs):
        """
        Send single notification message
        :param kwargs:
        :return:
        """
        # forming payload
        dump = kwargs.get('dump', False)
        payload = self._set_method_payload(**kwargs)

        # forming metadata for db
        if metadata := kwargs.get('metadata', {}):
            metadata['title'] = str(metadata.get('title'))
            metadata['description'] = str(metadata.get('description'))

        if dump or not payload.get('recipient'):
            return payload

        data = json.dumps(payload)
        params = {'url': self.URL_POST_WAITING, 'data': data}
        with suppress(ValidationError):
            response = self.make_request(method='post', **params)
            return self._save(response=response, metadata=metadata)

        return

    def bulk_send(self, notifications: list = None):
        """
        Send multiple notification message
        :param notifications:
        :return:
        """
        if not notifications:
            return []

        data = json.dumps({'notifications': notifications})
        params = {'url': self.URL_POST_WAITING_BULK, 'data': data}

        with suppress(ValidationError):
            response = self.make_request(method='post', **params)

            # Saving
            elements = []
            notifications = response.get('notifications', [])
            for notification in notifications:
                elements.append(self._save(response=notification))

            return elements

        return

    @staticmethod
    def _save(**kwargs):
        response = kwargs.get('response', {})
        delivery_method = response.get('delivery_method', 'email').upper()
        metadata = kwargs.get('metadata', {})

        # Object saving
        output = {
            'title': response.get('message_subject', ''),
            'description': response.get('message_body', ''),
            'read': response.get('is_read', False),
            'timestamp': response.get('created_at', datetime.now()),
            'delivery_method': delivery_method,
            'edited_timestamp': response.get('modified_at', datetime.now()),
            'recipient': response.get('recipient', ''),
            'sender': response.get('sender', ''),
            'code_name': response.get('id', ''),
            'type': response.get('delivery_method', 'email').upper(),
            'data': response,
            'status': NotificationHelper.STATUS_NOTIFIED,
            'metadata': metadata
        }

        return output

    def auth(self):
        self.headers['Authorization'] = f'Bearer {self.key}'
        return self



import os

from datetime import datetime
from contextlib import suppress

from rest_framework.exceptions import ValidationError

from ..helpers.attachment import AttachmentHelper
from ..services.abstract import AbstractServiceProvider

from ..config.settings import (SERVICE_ATTACHMENT_HOST, SERVICE_ATTACHMENT_SECRET_TOKEN, SERVICE_SSO_SECRET_KEY)


# Create your services here.

class AttachmentService(AbstractServiceProvider):
    URL_POST_FILE = 'file/upload/'

    def __init__(self):
        super().__init__()

        self.host = SERVICE_ATTACHMENT_HOST
        self.key = SERVICE_ATTACHMENT_SECRET_TOKEN
        self.secret = SERVICE_SSO_SECRET_KEY
        self.headers = {}

        # Authorize
        self.auth()

    def set_key(self, key: str = ''):
        self.key = key
        self.auth()
        return self

    @staticmethod
    def delete_file(path):
        os.remove(path)

    def upload_bulk(self, **kwargs):
        """
        Upload bulk files
        :param kwargs:
        :return:
        """
        files = kwargs.get('files', [])
        key = kwargs.get('key')
        typed = kwargs.get('type', AttachmentHelper.TYPE_OTHER)
        entities = kwargs.get('entities', False)

        if not files:
            return

        if key:
            self.set_key(key=key)

        elements = []
        for file in files:
            elements.append(self.upload(**{'files': file, 'key': key, 'type': typed, 'entities': entities}))

        return elements

    def upload(self, **kwargs):
        """
        Upload single files
        :param kwargs:
        :return:
        """
        files = kwargs.get('files')
        key = kwargs.get('key')
        typed = kwargs.get('type', AttachmentHelper.TYPE_OTHER)
        entities = kwargs.get('entities', False)
        tuples = kwargs.get('tuples', False)

        # Files not uploaded , skip
        if not files:
            return

        # Key specified update authorization
        if key:
            self.set_key(key=key)

        if isinstance(files, list) and len(files):
            return self.upload_bulk(**kwargs)

        # Remove content type for files
        self.headers.pop('content-type', None)
        files = {'file': files}
        params = {'url': self.URL_POST_FILE, 'files': files}

        with suppress(ValidationError):
            response = self.make_request(method='post', **params)
            response.update({'type': typed})
            # Saving
            element = self._save(response=response)
            return element if not entities else self.indexed(element, index=0, retrieve=True)

        return None if not tuples else None, False

    def upload_path(self, **kwargs):
        """
        Upload file on path
        :param kwargs:
        :return:
        """

        # Files not uploaded , skip
        if not (path := kwargs.get('path')):
            return

        if not os.path.exists(path):
            return

        with open(path, 'rb') as file:
            self.system()

            # Remove content type for files
            self.headers.pop('content-type', None)
            files = {'file': file}
            params = {'url': self.URL_POST_FILE, 'files': files}
            response = self.make_request(method='post', **params)
            response.update({'type': kwargs.get('type', AttachmentHelper.TYPE_OTHER)})

        # Remove local
        if kwargs.get('delete', True, ):
            os.remove(path)

        return self._save(response=response)

    @staticmethod
    def _save(**kwargs):
        response = kwargs.get('response', {})
        # Object saving
        attributes = response.get('attributes', {})
        output = {
            'url': response.get('url', ''),
            'file_name': response.get('file_hash', ''),
            'name': response.get('name', ''),
            'size': response.get('size', 0),
            'timestamp': response.get('created_at', datetime.now()),
            'edited_timestamp': response.get('created_at', datetime.now()),
            'width': attributes.get('width', 0),
            'height': attributes.get('height', 0),
            'mime_type': response.get('content_type', ''),
            'data': response,
            'type': response.get('type')
        }
        return output

    def auth(self):
        self.headers['Authorization'] = f'Bearer {self.key}'
        return self

    def system(self):
        self.headers['Authorization'] = f'Bearer {self.secret}'
        return self

    @staticmethod
    def indexed(elements=None, index: int = 0, retrieve: bool = False):
        """
        Get and / or retrieve element at index
        :param elements:
        :param index:
        :param retrieve:
        :return:
        """
        if elements is None:
            elements = []

        condition = index < len(elements)

        if condition and retrieve:
            return elements[index]

        return condition



class NotificationHelper:
    # STATUS
    STATUS_NOTIFIED = 'NOTIFIED'
    STATUS_OTHER = 'OTHER'

    STATUS_CHOICES = (
        (STATUS_NOTIFIED, STATUS_NOTIFIED),
        (STATUS_OTHER, STATUS_OTHER),
    )



import os

from pathlib import Path
from dotenv import load_dotenv

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent.parent.parent.parent.parent

env_path = BASE_DIR / '.env'
load_dotenv(dotenv_path=env_path, verbose=True)

# SSO Service
SERVICE_SSO_DOMAIN = os.getenv("SERVICE_SSO_DOMAIN")
SERVICE_SSO_SECRET_TOKEN = os.getenv("SERVICE_SSO_SECRET_TOKEN")
SERVICE_SSO_SERVICE_TOKEN = os.getenv("SERVICE_SSO_SERVICE_TOKEN")
SERVICE_SSO_SECRET_KEY = os.getenv("SERVICE_SSO_SECRET_KEY")

# Attachment
SERVICE_ATTACHMENT_HOST = os.getenv("SERVICE_ATTACHMENT_HOST")
SERVICE_ATTACHMENT_SECRET_KEY = os.getenv("SERVICE_ATTACHMENT_SECRET_KEY")
SERVICE_ATTACHMENT_SECRET_TOKEN = os.getenv("SERVICE_ATTACHMENT_SECRET_TOKEN")

# Notification
SERVICE_NOTIFICATION_HOST = os.getenv("SERVICE_NOTIFICATION_HOST")
SERVICE_NOTIFICATION_SECRET_KEY = os.getenv("SERVICE_NOTIFICATION_SECRET_KEY")



import requests

from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import ValidationError
from rest_framework.status import (HTTP_200_OK, HTTP_202_ACCEPTED, HTTP_201_CREATED)

from abc import ABC, abstractmethod
from contextlib import suppress

from simplejson import JSONDecodeError


class AbstractServiceProvider(ABC):
    default_headers = {
        'content-type': 'application/json',
        'charset': 'UTF-8',
        'connection': 'keep-alive',
        'accept': '*/*'
    }

    def __init__(self):
        self.host = None
        self.key = None
        self.username = None
        self.password = None
        self.token = None
        self.agent = None
        self.soap = None
        self.secret = None
        self.headers = self.default_headers

        # Auth service
        self.auth()

    @abstractmethod
    def auth(self) -> dict:
        """
            Base authentication.
            :param self: Class instance.
            :return: dict response data.
        """
        ...

    def create_request(self, method: str, **kwargs):
        """
        Make a request but return response instance
        :param method:
        :param kwargs:
        :return:
        """
        url = kwargs.pop('url')
        params = {'url': f'{self.host}{url}', 'headers': self.headers, 'allow_redirects': True}
        params.update(kwargs)

        # Where requests is a package, method are 'get' or 'post'
        response = getattr(requests, method)(**params)

        if response.status_code not in [HTTP_200_OK, HTTP_202_ACCEPTED, HTTP_201_CREATED]:
            detail = {}
            validation = ValidationError(detail=detail, code=response.status_code)

            with suppress(JSONDecodeError):
                validation.detail.update(**response.json())
                raise validation

            validation.detail = {
                'code': response.status_code,
                'detail': _('Remote provider failure.'),
                'content_type': response.headers['content-type'],
                'content': response.content,
            }

            raise validation

        return response

    def make_request(self, method: str, **kwargs):
        """
        Make a request but return json
        :param method:
        :param kwargs:
        :return:
        """
        response = self.create_request(method, **kwargs)
        return response.json()



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="drf-ebs-utils",
    version="0.0.22",
    author="Dorin Musteata",
    author_email="dorin.musteata@ebs-integrator.com",
    description="json2htmls is a package that converts any json to html tables.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ebs-integrator/json2htmls",
    project_urls={
        "Bug Tracker": "https://github.com/ebs-integrator/json2htmls/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
)



import json

from abc import ABC

from ..services.abstract import AbstractServiceProvider

from ..config.settings import (SERVICE_SSO_SERVICE_TOKEN, SERVICE_SSO_SECRET_TOKEN, SERVICE_SSO_DOMAIN)


# Create your services here.

class SSOService(AbstractServiceProvider, ABC):
    URL_POST_REGISTER = 'authentication/register'
    URL_POST_CONFIRM = 'users/confirm'
    URL_POST_LOGIN = 'authentication/login'
    URL_PATCH_RESEND = 'users/resend-confirmation'
    URL_PATCH_PASSWORD = 'users/set-password'
    URL_POST_RESTORE_PASSWORD = 'authentication/restore-request'
    URL_POST_RESTORE_CONFIRM_PASSWORD = 'authentication/restore-confirm'
    URL_POST_REFRESH = 'authentication/refresh'

    def __init__(self):
        super().__init__()

        self.host = SERVICE_SSO_DOMAIN
        self.key = SERVICE_SSO_SECRET_TOKEN
        self.token = SERVICE_SSO_SERVICE_TOKEN

        # Authorize
        self.auth()

    def profile(self, **kwargs):
        """
        Profile user
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        params = {'url': self.URL_POST_LOGIN, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def login(self, **kwargs):
        """
        Register user
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_LOGIN, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def set_password(self, **kwargs):
        """
        Set user password
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})
        headers = kwargs.get('headers', {})
        authorization = headers.get('Authorization', '')

        data = json.dumps(payload)
        self.headers.update({'Authorization': authorization})
        params = {'url': self.URL_PATCH_PASSWORD, 'data': data}
        response = self.make_request(method='patch', **params)

        return response

    def reset_password(self, **kwargs):
        """
        Reset user password
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_RESTORE_PASSWORD, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def reset_password_confirm(self, **kwargs):
        """
        Reset user password confirm
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_RESTORE_CONFIRM_PASSWORD, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def refresh(self, **kwargs):
        """
        Refresh user
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_REFRESH, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def register(self, **kwargs):
        """
        Register user
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_REGISTER, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def confirm(self, **kwargs):
        """
        Confirm user
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_POST_CONFIRM, 'data': data}
        response = self.make_request(method='post', **params)

        return response

    def resend(self, **kwargs):
        """
        Resend user confirmation
        :param kwargs:
        :return:
        """
        payload = kwargs.get('data', {})

        data = json.dumps(payload)
        self.headers.pop('Authorization', None)
        params = {'url': self.URL_PATCH_RESEND, 'data': data}
        response = self.make_request(method='patch', **params)

        return response

    def auth(self):
        self.headers['Service-Token'] = f'{self.token}'
        self.headers['Secret-Token'] = f'{self.key}'
        return self
